// DaoGrid.cpp : implementation file
//

#include "stdafx.h"
#include "resource.h"
#include "ADOGrid.h"
#include <comdef.h>
#include <math.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


#define ID_TIMER_COLWIDTHS			10
#define TIME_COLWIDTHS				2000


/////////////////////////////////////////////////////////////////////////////
// Global functions


/////////////////////////////////////////////////////////////////////////////
// CADOGrid

CBrush CADOGrid::m_brshWnd;
CFont  CADOGrid::m_fnt;

CADOGrid::CADOGrid()
{
	if (!HBRUSH(m_brshWnd))
	{
		m_brshWnd.CreateSolidBrush(GetSysColor(COLOR_WINDOW));

		CWindowDC dc(CWnd::GetDesktopWindow());
		CFont *pOld = (CFont *)dc.SelectStockObject(ANSI_VAR_FONT);
		pOld = dc.SelectObject(pOld);
		LOGFONT lf;
		pOld->GetLogFont(&lf);
		m_fnt.CreateFontIndirect(&lf);
	}

	CWindowDC dc(CWnd::GetDesktopWindow());
	CFont *pOld = (CFont *)dc.SelectStockObject(ANSI_VAR_FONT);
	m_nTextCy = dc.GetTextExtent("W").cy + 2;
	dc.SelectObject(pOld);

	m_pRecSet = 0;
	m_pDC = 0;
	m_nTextCx = 0;
	m_bBound = FALSE;
	m_bOwner = TRUE;

	m_nScrollX = 0;
	m_nScrollY = 0;
	m_nSelectedRow = 0;
	m_pImages = 0;

	m_bCtxFindOn = TRUE;
	m_hCtxWnd = NULL;
}

CADOGrid::~CADOGrid()
{
	CleanUp();
}


BEGIN_MESSAGE_MAP(CADOGrid, CWnd)
	//{{AFX_MSG_MAP(CADOGrid)
	ON_WM_SIZE()
	ON_WM_PAINT()
	ON_WM_HSCROLL()
	ON_WM_VSCROLL()
	ON_WM_SETFOCUS()
	ON_WM_KILLFOCUS()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONDBLCLK()
	ON_WM_RBUTTONUP()
	ON_NOTIFY(HDN_TRACK, 1, OnHdrTrack)
	ON_NOTIFY(HDN_BEGINTRACK, 1, OnHdrTrack)
	ON_NOTIFY(HDN_ENDTRACK, 1, OnHdrTrack)
	ON_WM_MOUSEWHEEL()
	//}}AFX_MSG_MAP
	ON_NOTIFY(HDN_ITEMCLICK, 1, OnHdrColumnClick)
	ON_NOTIFY(HDN_DIVIDERDBLCLICK, 1, OnHdrDividerClick)
END_MESSAGE_MAP()

BOOL CADOGrid::Create(CWnd *pWndParent, int nId, int nStyle, int nExtStyle, const CRect &rect)
{
	// Register window class first
	CString strWndClass = AfxRegisterWndClass(CS_DBLCLKS, ::LoadCursor(NULL, IDC_ARROW), m_brshWnd);

	// Check style
	nStyle |= WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN | WS_VSCROLL | WS_HSCROLL | WS_TABSTOP;

	// Create actual window then
	if (!CWnd::CreateEx(nExtStyle, strWndClass, NULL, nStyle, rect, pWndParent, nId))
		return FALSE;
	SetFont(&m_fnt);

	// Create header control
	if (!m_hdr.Create(WS_CHILD | WS_VISIBLE | HDS_BUTTONS | HDS_HORZ, CRect(0,0,rect.right,10), this, 1))
		return FALSE;
	m_hdr.SetFont(&m_fnt);
	LayoutHdr();
	AdjustScrolling();

	return TRUE;
}

void CADOGrid::LayoutHdr()
{
	CRect rcClient;
	GetClientRect(rcClient);

	HD_LAYOUT  l;
	WINDOWPOS wp;
	l.prc = &rcClient;
	l.pwpos = &wp;
	m_hdr.Layout(&l);
	m_hdr.SetWindowPos(CWnd::FromHandle(wp.hwndInsertAfter), 
					   wp.x - m_nScrollX, wp.y, wp.cx + m_nScrollX, wp.cy, wp.flags);
	m_nOrigY = wp.cy;
	m_sizeGrid.cx = rcClient.right;
	m_sizeGrid.cy = rcClient.bottom - m_nOrigY;
}

/////////////////////////////////////////////////////////////////////////////
// CADOGrid message handlers

void CADOGrid::OnSize(UINT nType, int cx, int cy) 
{
	CWnd::OnSize(nType, cx, cy);
	
	// Update position of header control
	if (m_hdr.m_hWnd)
	{
		LayoutHdr();
		AdjustScrolling();
	}
}

void CADOGrid::CleanUp()
{
//	if (m_bOwner)
//		if (m_pRecSet)
//			delete m_pRecSet;

	m_pRecSet = 0;
	m_bBound = FALSE;
	m_bOwner = TRUE;
	m_aColumns.RemoveAll();

	m_nScrollX = m_nScrollY = 0;
	m_nSelectedRow = 0;
	if (m_hdr.m_hWnd)
		while (m_hdr.GetItemCount())
			m_hdr.DeleteItem(0);

	if (GetSafeHwnd())
		Invalidate();
}


void CADOGrid::OpenQuery(_RecordsetPtr rs, const CStringArray &aColumns,
		           const CStringArray &aColNames)
{
	ASSERT(aColumns.GetSize());
	ASSERT(aColumns.GetSize() == aColNames.GetSize());

	// Close old recordset
	CleanUp();

	// Open new one
	m_pRecSet = rs;
	
	if(!m_pRecSet->adoEOF)
	{
		m_bBound = TRUE;
		m_nRecCount=0;
		while(!m_pRecSet->adoEOF)
		{
			m_pRecSet->MoveNext();
			m_nRecCount++;
		}
		m_pRecSet->MoveFirst();

//		m_nRecCount=m_pRecSet->RecordCount;
	}

	// DC for text sizes
	CClientDC dc(this);
	CFont *pOld = dc.SelectObject(&m_fnt);

	// Bind selected columns
	HD_ITEM hi;
	for (int i = 0; i < aColumns.GetSize(); i++)
	{
		hi.mask = HDI_FORMAT | HDI_TEXT | HDI_WIDTH;
		hi.cxy = dc.GetTextExtent(aColNames[i]).cx + 14;
		hi.pszText = strdup(aColNames[i]);
		hi.cchTextMax = aColumns[i].GetLength();
		hi.fmt = HDF_LEFT | HDF_STRING;
		m_hdr.InsertItem(m_hdr.GetItemCount(), &hi);
		free(hi.pszText);

		m_aColumns.Add(aColumns[i]);
	}
	dc.SelectObject(pOld);

	ScrollTo(0, 0);
	CalcTextCx();
	AdjustScrolling();
	Invalidate();
}



void CADOGrid::ReQuery(const CString &strSql)
{
	ASSERT(m_bBound);
	ASSERT(m_bOwner);

/*	m_nSelectedRow = 0;
	SetCurPos(0);
	m_pRecSet->Close();
	m_pRecSet->Open(dbOpenDynaset, strSql);

	if (!m_pRecSet->IsEOF())
	{
		m_pRecSet->MoveLast();
		m_pRecSet->MoveFirst();
	}
	else
		m_bBound = FALSE;
*/
	AdjustScrolling();
	Invalidate();
}

void CADOGrid::BindReQuery(BOOL nSetPos)
{
//	ASSERT(!m_bOwner);

	m_pRecSet->MoveFirst();
	m_nRecCount=0;
	m_bBound = FALSE;
	while(!m_pRecSet->adoEOF)
	{
		m_bBound = TRUE;
		m_pRecSet->MoveNext();
		m_nRecCount++;
	}
	if(m_bBound)
		m_pRecSet->MoveFirst();

	if(nSetPos)
	{
		m_nSelectedRow = 0;
		if (m_bBound)
			SetCurPos(0);
	}

	AdjustScrolling();
	Invalidate();
}

void CADOGrid::OnHdrColumnClick(NMHDR *pNMHDR, LRESULT *pResult)
{
	// Notify parent window
	CWnd *pWndParent = GetParent();
	if (pWndParent && m_bBound)
	{
		NM_ADOGRID nmg;
		nmg.nmhdr.hwndFrom = GetSafeHwnd();
		nmg.nmhdr.idFrom = GetDlgCtrlID();
		nmg.nmhdr.code = DGN_COLUMNCLICK;
		nmg.iRow = ((HD_NOTIFY *)pNMHDR)->iItem;
		nmg.strCol = m_aColumns[nmg.iRow];
		
		pWndParent->SendMessage(WM_NOTIFY, GetDlgCtrlID(), (LPARAM)(void *)&nmg);
	}
}

void CADOGrid::OnHdrDividerClick(NMHDR *pNMHDR, LRESULT *pResult)
{
	// Notify parent window
	CWnd *pWndParent = GetParent();
	if (pWndParent && m_bBound)
	{
		NM_ADOGRID nmg;
		nmg.nmhdr.hwndFrom = GetSafeHwnd();
		nmg.nmhdr.idFrom = GetDlgCtrlID();
		nmg.nmhdr.code = DGN_DIVIDERDBL;
		nmg.iRow = ((HD_NOTIFY *)pNMHDR)->iItem;
		
		pWndParent->SendMessage(WM_NOTIFY, GetDlgCtrlID(), (LPARAM)(void *)&nmg);
	}


}

void CADOGrid::OnHdrTrack(NMHDR* pNMHDR, LRESULT* pResult) 
{
	HD_NOTIFY *phdn = (HD_NOTIFY *) pNMHDR;
	static CRect rcClient;

	switch (phdn->hdr.code)
	{
	case HDN_BEGINTRACK:
		if (!m_pDC)
		{
			m_pDC = new CClientDC(this);
			m_pDC->SetWindowOrg(m_nScrollX, 0);
			GetClientRect(&rcClient);
			m_nCy = rcClient.bottom;
			m_nCx = GetColumnLeftPos(phdn->iItem);
			m_nOldCx = m_nLastCx = phdn->pitem->cxy;

			m_pDC->SetROP2(R2_NOT);
			m_pDC->MoveTo(m_nCx + m_nLastCx, 0);
			m_pDC->LineTo(m_nCx + m_nLastCx, m_nCy);
		}
		break;

	case HDN_TRACK:
		if (m_pDC)
		{
			m_pDC->MoveTo(m_nCx + m_nLastCx, 0);
			m_pDC->LineTo(m_nCx + m_nLastCx, m_nCy);
			m_nLastCx = phdn->pitem->cxy;
			m_pDC->MoveTo(m_nCx + m_nLastCx, 0);
			m_pDC->LineTo(m_nCx + m_nLastCx, m_nCy);
		}
		break;

	case HDN_ENDTRACK:
		if (m_pDC)
		{
			m_pDC->MoveTo(m_nCx + m_nLastCx, 0);
			m_pDC->LineTo(m_nCx + m_nLastCx, m_nCy);
			delete m_pDC;
			m_pDC = 0;

			// Scroll window
			Invalidate();
			m_nTextCx += phdn->pitem->cxy - m_nOldCx;
			AdjustScrolling();
			LayoutHdr();
		}
		break;
	}
	
	*pResult = 0;
}

int CADOGrid::GetColumnLeftPos(int index)
{
	int cx = 0;
	HD_ITEM hi;
	for (int i = 0; i < index; i++)
	{
		hi.mask = HDI_WIDTH;
		m_hdr.GetItem(i, &hi);
		cx += hi.cxy;
	}
	return cx;
}

void CADOGrid::CalcTextCx()
{
	m_nTextCx = 0;
	HD_ITEM hi;
	for (int i = 0; i < m_hdr.GetItemCount(); i++)
	{
		hi.mask = HDI_WIDTH;
		m_hdr.GetItem(i, &hi);
		m_nTextCx += hi.cxy;
	}
}

void CADOGrid::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	if (!m_bBound)
		return;
//	ASSERT(m_pRecSet);
	
	// Setup origin and others
	dc.SetViewportOrg(-m_nScrollX, m_nOrigY);
	dc.SetBkMode(TRANSPARENT);
	CFont *pOld = dc.SelectObject(&m_fnt);
	CalcTextCx();

	// Draw all rows
	_variant_t varValue;
	CRect       rcClient;
	CRect       rcClip;
	CRect       rcTemp;
	GetClientRect(&rcClient);
	dc.GetClipBox(&rcClip);

	int nRows = min(m_nScrollY + m_sizeGrid.cy / m_nTextCy + 1, m_nRecCount);
	for (int nRow = m_nScrollY; nRow < nRows; nRow++)
	{
		// Position in recordset _RecordsetPtr
		if (m_pRecSet->adoEOF || m_pRecSet->BOF)
			return;
		m_pRecSet->MoveFirst();
		m_pRecSet->Move(nRow);

		BOOL bHilightColor = FALSE;
		CRect rcItem = GetRowRect(nRow);
		if (!rcTemp.IntersectRect(rcItem, rcClip))
			continue;
		
		// Draw selected border if selection
		if (nRow == m_nSelectedRow)
		{
			if (m_pImages)
				rcItem.left += m_rcImage.Width() + 1;

			dc.FillRect(rcItem, &CBrush(::GetSysColor(COLOR_HIGHLIGHT)));
			if (GetFocus() == this)
				dc.DrawFocusRect(&rcItem);
			bHilightColor = TRUE;
		}
		else
		{
			if (m_pImages)
				rcItem.left += m_rcImage.Width() + 1;
			rcItem.bottom+=1;
		}

		// Draw all columns
		HD_ITEM hi;
		int     nXPos = 0;
		for (int ic = 0; ic < m_hdr.GetItemCount(); ic++)
		{
			// Get width
			memset(&hi, 0, sizeof(HD_ITEM));
			hi.mask = HDI_WIDTH | HDI_FORMAT;
			m_hdr.GetItem(ic, &hi);

			// Calc item rect
			CRect rcText(nXPos-1,rcItem.top, nXPos + hi.cxy,rcItem.bottom);
			dc.DrawEdge(rcText,BDR_RAISEDOUTER,BF_RECT);
			rcText.left+=2;	rcText.top+=1;rcText.right-=4;rcText.bottom-=1;
			if (!rcTemp.IntersectRect(rcText, rcClip))
			{
				nXPos += hi.cxy;
				continue;
			}
			// Translate field value to text
//			CString strText ;//= OnFormatValue(nRow, ic, m_aColumns[ic], varValue, bHilightColor);
			varValue = m_pRecSet->Fields->GetItem(_variant_t(m_aColumns[ic]))->Value;
			CString strText;
			if(varValue.vt==1)
				strText="";
			else
				strText= GETSTRING(varValue);

			// Get text color
			COLORREF color = OnFormatColor(nRow, ic, m_aColumns[ic], bHilightColor);

			// Get image
			if (!ic && m_pImages)
			{
				int nImage = 0;//OnFormatImage(nRow, bHilightColor);
				if (nImage != -1)
					m_pImages->Draw(&dc, nImage, rcText.TopLeft(), ILD_TRANSPARENT);

				rcText.left += m_rcImage.Width() + 1;
			}

			// Draw text
			dc.SetTextColor(color);
			int nAlign;
			if (hi.fmt & HDF_CENTER)
				nAlign = DT_CENTER;
			else if (hi.fmt & HDF_RIGHT)
				nAlign = DT_RIGHT;
			else
				nAlign = DT_LEFT;
			dc.DrawText(strText, &rcText, nAlign | DT_NOPREFIX | DT_SINGLELINE | DT_END_ELLIPSIS);
			dc.SetTextColor(0);

			nXPos += hi.cxy;
		}
	}

	dc.SelectObject(pOld);
}

CRect CADOGrid::GetRowRect(int nRow)
{
	ASSERT(m_bBound);
//	ASSERT(nRow >= 0 && nRow < m_nRecCount);

	CRect rcClient;
	GetClientRect(&rcClient);
	return CRect(0, (nRow - m_nScrollY) * m_nTextCy, m_nTextCx, (nRow - m_nScrollY + 1) * m_nTextCy);
}

void CADOGrid::AdjustScrolling()
{
	SCROLLINFO si;
	memset(&si, 0, sizeof(si));
	si.cbSize = sizeof(SCROLLINFO);
	si.fMask = SIF_PAGE | SIF_RANGE;
	si.nMin = 0;
	si.nMax = m_nTextCx;
	si.nPage = m_sizeGrid.cx;
	SetScrollInfo(SB_HORZ, &si);

	si.nMin = 0;
	si.nMax = (m_bBound) ? m_nRecCount - 1 : 0;
	si.nPage = m_sizeGrid.cy / m_nTextCy;
	SetScrollInfo(SB_VERT, &si);

	si.fMask |= SIF_POS;
	GetScrollInfo(SB_HORZ, &si);
	m_nScrollX = si.nPos;

	si.fMask |= SIF_POS;
	GetScrollInfo(SB_VERT, &si);
	m_nScrollY = si.nPos;
}

void CADOGrid::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	SCROLLINFO si;
	memset(&si, 0, sizeof(si));
	si.cbSize = sizeof(SCROLLINFO);
	si.fMask = SIF_ALL;
	GetScrollInfo(SB_HORZ, &si);

	int nDelta;
	switch (nSBCode)
	{
	case SB_PAGERIGHT:
		nDelta = si.nPage;
		break;

	case SB_PAGELEFT:
		nDelta = -(int)si.nPage;
		break;

	case SB_LINERIGHT:
		nDelta = 25;
		break;
	
	case SB_LINELEFT:
		nDelta = -25;
		break;

	case SB_THUMBPOSITION:
	case SB_THUMBTRACK:
		nDelta = si.nTrackPos - si.nPos;
		break;

	default:
		nDelta = 0;
		break;
	}

	if (nDelta)
		ScrollTo(si.nPos + nDelta, m_nScrollY);
}

void CADOGrid::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	SCROLLINFO si;
	memset(&si, 0, sizeof(si));
	si.cbSize = sizeof(SCROLLINFO);
	si.fMask = SIF_ALL;
	GetScrollInfo(SB_VERT, &si);

	int nDelta;
	switch (nSBCode)
	{
	case SB_PAGERIGHT:
		nDelta = si.nPage;
		break;

	case SB_PAGELEFT:
		nDelta = -(int)si.nPage;
		break;

	case SB_LINERIGHT:
		nDelta = 1;
		break;
	
	case SB_LINELEFT:
		nDelta = -1;
		break;

	case SB_THUMBPOSITION:
	case SB_THUMBTRACK:
		nDelta = si.nTrackPos - si.nPos;
		break;

	default:
		nDelta = 0;
		break;
	}

	if (nDelta)
		ScrollTo(m_nScrollX, si.nPos + nDelta);
}

void CADOGrid::ScrollTo(int newX, int newY)
{
	SCROLLINFO six, siy;
	memset(&six, 0, sizeof(six));
	six.cbSize = sizeof(SCROLLINFO);
	six.fMask = SIF_ALL;
	GetScrollInfo(SB_HORZ, &six);
	memset(&siy, 0, sizeof(siy));
	siy.cbSize = sizeof(SCROLLINFO);
	siy.fMask = SIF_ALL;
	GetScrollInfo(SB_VERT, &siy);

	int nOldX = six.nPos;
	six.nPos = newX;
	SetScrollInfo(SB_HORZ, &six);
	GetScrollInfo(SB_HORZ, &six);
	m_nScrollX = six.nPos;

	int nOldY = siy.nPos;
	siy.nPos = newY;
	SetScrollInfo(SB_VERT, &siy);
	GetScrollInfo(SB_VERT, &siy);
	m_nScrollY = siy.nPos;

	ScrollWindow(nOldX - six.nPos, (nOldY - siy.nPos) * m_nTextCy, &CRect(0, m_nOrigY, m_sizeGrid.cx, m_sizeGrid.cy + m_nOrigY));
	LayoutHdr();
}

BOOL CADOGrid::IsVisible(int nRow)
{
	ASSERT(m_bBound);
	ASSERT(nRow >= 0 && nRow < m_nRecCount);
	return (nRow <= m_nScrollY + m_sizeGrid.cy / m_nTextCy - 1 &&
			nRow >= m_nScrollY);
}

void CADOGrid::EnsureVisible(int nRow)
{
	ASSERT(m_bBound);
	ASSERT(nRow >= 0 && nRow < m_nRecCount);

	// Is nRow too down?
	if (nRow > m_nScrollY + m_sizeGrid.cy / m_nTextCy - 1)
		ScrollTo(m_nScrollX, nRow - m_sizeGrid.cy / m_nTextCy + 1);
	else if (nRow < m_nScrollY) // Too up
		ScrollTo(m_nScrollX, nRow);
}

BOOL CADOGrid::PreTranslateMessage(MSG* pMsg) 
{
	// Navigation support
	if (pMsg->message == WM_KEYDOWN && m_bBound)
	{
		switch (pMsg->wParam)
		{
		case VK_UP:
			if (m_nSelectedRow)
				SetCurPos(m_nSelectedRow - 1);
			else
				MessageBeep(-1);
			return TRUE;

		case VK_DOWN:
			if (m_nSelectedRow < m_nRecCount - 1)
				SetCurPos(m_nSelectedRow + 1);
			else
				MessageBeep(-1);
			return TRUE;

		case VK_PRIOR:
			if (m_nSelectedRow)
				SetCurPos(max(m_nSelectedRow - (m_sizeGrid.cy / m_nTextCy - 1), 0));
			else
				MessageBeep(-1);
			return TRUE;

		case VK_NEXT:
			if (m_nSelectedRow < m_nRecCount - 1)
			{
				SetCurPos(min(m_nSelectedRow + m_sizeGrid.cy / m_nTextCy - 1, 
							  m_nRecCount - 1));
			}
			else
				MessageBeep(-1);
			return TRUE;

		case VK_HOME:
			if (m_nSelectedRow)
			{
				SetCurPos(0);
			}
			else
				MessageBeep(-1);
			return TRUE;

		case VK_END:
			if (m_nSelectedRow < m_nRecCount - 1)
			{
				SetCurPos(m_nRecCount - 1);
			}
			else
				MessageBeep(-1);
			return TRUE;

		default:
			break;
		}
	}

	// Context search support
	if (m_bCtxFindOn && m_bBound &&
		pMsg->message == WM_CHAR || (pMsg->message == WM_KEYDOWN && pMsg->wParam == VK_BACK))
	{
		// Get current find str
		BOOL bDown = TRUE;
		CString strText = m_strCtxText;
		if (pMsg->message == WM_KEYDOWN)
		{
			if (strText.IsEmpty())
			{
				MessageBeep(-1);
				return TRUE;
			}
			else
				strText = strText.Left(strText.GetLength() - 1);
			bDown = FALSE;
		}
		else
			strText += (char)pMsg->wParam;

		// Try to find next occurence
		int nNewIndex;
		if (strText.IsEmpty())
			nNewIndex = 0;
		else
			nNewIndex = FindItem(strText, bDown);
		if (nNewIndex != -1)
		{
			m_strCtxText = strText;
			if (m_nSelectedRow != nNewIndex) SetCurPos(nNewIndex, FALSE);
			if (m_hCtxWnd)
				CWnd::FromHandle(m_hCtxWnd)->SetWindowText(m_strCtxText);
		}
		else
			MessageBeep(-1);

		return TRUE;
	}
		
	return CWnd::PreTranslateMessage(pMsg);
}

void CADOGrid::OnSetFocus(CWnd* pOldWnd) 
{
	CWnd::OnSetFocus(pOldWnd);

	if (m_bBound)
	{
		EnsureVisible(m_nSelectedRow);
		CRect rc = GetRowRect(m_nSelectedRow);
		rc.OffsetRect(0, m_nOrigY);
		InvalidateRect(&rc, FALSE);
	}
}

void CADOGrid::OnKillFocus(CWnd* pNewWnd) 
{
	CWnd::OnKillFocus(pNewWnd);
	
	if (m_bBound)
		if (IsVisible(m_nSelectedRow))
		{
			CRect rc = GetRowRect(m_nSelectedRow);
			rc.OffsetRect(0, m_nOrigY);
			InvalidateRect(&rc, FALSE);
		}
}


long CADOGrid::GetFieldValueLong(const CString &strName)
{
	ASSERT(m_bBound);
	// Position recordset
	m_pRecSet->MoveFirst();
	m_pRecSet->Move(m_nSelectedRow);
	_variant_t varValue = m_pRecSet->Collect[_variant_t(strName)];
	long nRet = (long)varValue;
	return nRet;
}


void CADOGrid::SetCurPos(int nRow, BOOL bClearFindText)
{
	ASSERT(m_bBound);
	ASSERT(nRow >= 0 && nRow < m_nRecCount);

	int o = m_nSelectedRow;
	m_nSelectedRow = nRow;
	EnsureVisible(m_nSelectedRow);
	
	CRect rc = GetRowRect(m_nSelectedRow);
	rc.OffsetRect(0, m_nOrigY);
	InvalidateRect(&rc);
	rc = GetRowRect(o);
	rc.OffsetRect(0, m_nOrigY);
	InvalidateRect(&rc);

	// Notify parent window
	CWnd *pWndParent = GetParent();
	if (pWndParent)
	{
		NM_ADOGRID nmg;
		nmg.nmhdr.hwndFrom = GetSafeHwnd();
		nmg.nmhdr.idFrom = GetDlgCtrlID();
		nmg.nmhdr.code = DGN_ITEMCHANGED;
		nmg.iRow = m_nSelectedRow;

		pWndParent->SendMessage(WM_NOTIFY, GetDlgCtrlID(), (LPARAM)(void *)&nmg);
	}

	if (bClearFindText)
	{
		m_strCtxText.Empty();
		if (m_hCtxWnd)
			CWnd::FromHandle(m_hCtxWnd)->SetWindowText("");
	}
}

void CADOGrid::SetImageList(CImageList *pList) 
{ 
	ASSERT(pList); 
	m_pImages = pList; 
	
	IMAGEINFO imf;
	pList->GetImageInfo(0, &imf);
	m_rcImage = imf.rcImage;
	m_nTextCy = max(m_nTextCy, m_rcImage.Height());
	Invalidate();
}

void CADOGrid::OnLButtonDown(UINT nFlags, CPoint point) 
{
	if (!m_bBound)
		return;
	if (point.y < m_nOrigY || point.x > m_nTextCx - m_nScrollX)
		return;

	int nRow = min(m_nScrollY + (point.y - m_nOrigY) / m_nTextCy, m_nRecCount - 1);
	SetCurPos(nRow);
	SetFocus();
}

void CADOGrid::OnRButtonUp(UINT nFlags, CPoint point) 
{
	OnLButtonDown(nFlags,point);
	ClientToScreen(&point);
	CWnd *pWndParent = GetParent();
	if (pWndParent && m_bBound)
	{
		NM_ADOGRID nmg;
		nmg.nmhdr.hwndFrom = GetSafeHwnd();
		nmg.nmhdr.idFrom = GetDlgCtrlID();
		nmg.nmhdr.code = DGN_RCLICK;
		nmg.pos.x=point.x;
		nmg.pos.y=point.y;
		nmg.iRow = m_nSelectedRow;

		pWndParent->SendMessage(WM_NOTIFY, GetDlgCtrlID(), (LPARAM)(void *)&nmg);
	}
}

void CADOGrid::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
	// Notify parent window
	CWnd *pWndParent = GetParent();
	if (pWndParent && m_bBound)
	{
		NM_ADOGRID nmg;
		nmg.nmhdr.hwndFrom = GetSafeHwnd();
		nmg.nmhdr.idFrom = GetDlgCtrlID();
		nmg.nmhdr.code = DGN_DBLCLICKED;
		nmg.iRow = m_nSelectedRow;

		pWndParent->SendMessage(WM_NOTIFY, GetDlgCtrlID(), (LPARAM)(void *)&nmg);
	}
}

void CADOGrid::SetColAlign(int nCol, int nAlign)
{
//	ASSERT(m_bBound);
	HD_ITEM hi;

	memset(&hi, 0, sizeof(hi));
	hi.mask = HDI_FORMAT;
	hi.fmt = nAlign | HDF_STRING;
	m_hdr.SetItem(nCol, &hi);
}

int CADOGrid::GetColWidth(int nCol)
{
	ASSERT(m_bBound);
	HD_ITEM hi;

	memset(&hi, 0, sizeof(hi));
	hi.mask = HDI_WIDTH;
	m_hdr.GetItem(nCol, &hi);
	return hi.cxy;
}

void CADOGrid::SetColWidth(int nCol, int nWidth)
{
//	ASSERT(m_bBound);
	HD_ITEM hi;

	memset(&hi, 0, sizeof(hi));
	hi.mask = HDI_WIDTH;
	hi.cxy = nWidth;
	m_hdr.SetItem(nCol, &hi);

	// Scroll window
	Invalidate();
	CalcTextCx();
	AdjustScrolling();
	LayoutHdr();
}


int CADOGrid::FindItem(const CString &str, BOOL bDown)
{
/*	if (!m_bBound || !m_bCtxFindOn)
		return -1;

	int nIndex = m_nSelectedRow;
	if (!bDown) nIndex = 0;

	CString strFind = m_aColumns[0] + " LIKE '" + str + "*'";
	BOOL bResult;

	if (nIndex)
	{
		m_pRecSet->SetAbsolutePosition(nIndex - 1);
		bResult = m_pRecSet->FindNext(strFind);
	}
	else
	{
		m_pRecSet->SetAbsolutePosition(0);
		bResult = m_pRecSet->FindFirst(strFind);
	}

	if (bResult)
		return m_pRecSet->SetAbsolutePosition(nRow); //m_pRecSet->GetAbsolutePosition();
		
	else*/
		return -1;
}


int CADOGrid::GetColCount()
{
	return m_aColumns.GetSize();
}

inline COLORREF CADOGrid::OnFormatColor(int, int, LPCTSTR, BOOL bHilightColor)
{
	return (bHilightColor) ? ::GetSysColor(COLOR_HIGHLIGHTTEXT) : 
	                         ::GetSysColor(COLOR_WINDOWTEXT);
}

void CADOGrid::Refresh()
{
	if(!m_pRecSet->adoEOF)
	{
		m_pRecSet->MoveFirst();
		m_nRecCount=0;
		m_bBound = TRUE;
		while(!m_pRecSet->adoEOF)
		{
			m_pRecSet->MoveNext();
			m_nRecCount++;
		}
		m_pRecSet->MoveFirst();
	}
	Invalidate();
}

void CADOGrid::OnPreparePrinting(CPrintInfo *pInfo)
{
//	pInfo->SetMinPage(1);
//	pInfo->SetMaxPage(50);
	m_isPrint = FALSE;
}

void CADOGrid::OnPrint(CDC *pDC, CPrintInfo *pInfo)
{
	pDC->SetMapMode(MM_TEXT);
	CFont   ff,mff;
	LOGFONT lf;
	mff.CreatePointFont(100,"Arial New",pDC);
	mff.GetLogFont(&lf);
	lf.lfCharSet=EASTEUROPE_CHARSET ; //EE;
	ff.CreateFontIndirect(&lf);
	CFont *pOldF = pDC->SelectObject(&ff);

	_variant_t varValue;
	CString strVal;
	CSize sVal;
	HD_ITEM hi;

	CSize sMarg(1000,0);
	pDC->HIMETRICtoDP(&sMarg);
	long nMargin = sMarg.cx;
	long nPageSize = (pInfo->m_rectDraw.right-pInfo->m_rectDraw.left-nMargin);

	// Calculate sizes of items
	if(!m_isPrint)
	{
		for (int ic = 0; ic < m_hdr.GetItemCount(); ic++)
			aWidths.Add(50);
		m_nPrintHeight = 0;

		m_pRecSet->MoveFirst();
		while(!m_pRecSet->adoEOF)
		{
			for (int ic = 0; ic < m_hdr.GetItemCount(); ic++)
			{
				varValue = m_pRecSet->Fields->GetItem(_variant_t(m_aColumns[ic]))->Value;
				if(varValue.vt==1)
					strVal="";
				else
					strVal= GETSTRING(varValue);
//				strVal = (char*)((_bstr_t)(varValue));
				//strVal = OnFormatValue(0, ic, m_aColumns[ic], varValue, FALSE ); //bHilightColor);
				sVal = pDC->GetTextExtent(strVal);
				if (aWidths.GetAt(ic) < sVal.cx)
				{
					if(sVal.cx > (pInfo->m_rectDraw.right-pInfo->m_rectDraw.left)*3/4)
					{
						aWidths[ic] = (pInfo->m_rectDraw.right-pInfo->m_rectDraw.left)*3/4;
					}
					else
					{
						aWidths[ic] = sVal.cx;
					}

				}
				if(m_nPrintHeight < sVal.cy)
					m_nPrintHeight = sVal.cy;
			}
			m_pRecSet->MoveNext();
		}
		CString strHead;
		for (ic = 0; ic < m_hdr.GetItemCount(); ic++)
		{
			memset(&hi, 0, sizeof(HD_ITEM));
			hi.pszText  = strHead.GetBuffer(255);
			hi.cchTextMax = 255;
			hi.mask = HDI_TEXT ;
			m_hdr.GetItem(ic, &hi);
			strHead.ReleaseBuffer();
			sVal = pDC->GetTextExtent(strHead);
			if (sVal.cx/3 > aWidths[ic])
			{
				aWidths[ic] = sVal.cx*4/10;
			}
			else
			{
				if(aWidths[ic] < sVal.cx)
					aWidths[ic] = sVal.cx;
			}
			aWidths[ic]+=12;
		}
		m_nPrintHeight += 12;
		long nCurSize=0,nCurCol=0;
		m_nPrintPages = 0;
		BOOL bFinish = TRUE;
		while(bFinish)
		{
			for (ic = nCurCol; ic < m_hdr.GetItemCount(); ic++)
			{
				nCurSize += aWidths[ic];
				if(nCurSize>nPageSize)
				{
					nCurSize=0;
					nCurCol = ic;
					m_nPrintPages++;
					break;
				}
				if(ic == m_hdr.GetItemCount() -1)
				{
					bFinish = FALSE;
					m_nPrintPages++;
				}
			}
		}

		m_nPrintHdr = 8;
		m_nPrintRows = (pInfo->m_rectDraw.bottom-pInfo->m_rectDraw.top)/m_nPrintHeight - m_nPrintHdr - 2;
		long nPages = ceil((double)m_pRecSet->GetRecordCount()/(double)m_nPrintRows);
		nPages*=m_nPrintPages;
		pInfo->SetMaxPage(nPages);
		m_isPrint = TRUE;
	}
	long nCurPage = (pInfo->m_nCurPage-1) % m_nPrintPages;
	long nCurCol=0,nCurSize=0;
	for (int id=0;id<nCurPage;id++)
	{
		for (int ic = nCurCol; ic < m_hdr.GetItemCount(); ic++)
		{
			nCurSize += aWidths[ic];
			if(nCurSize>nPageSize)
			{
				nCurSize=0;
				nCurCol = ic;
				break;
			}
		}
	}

	int     nXPos = nMargin;
	
	// print header
	CString strHead;
	CRect rcText(pInfo->m_rectDraw.left,m_nPrintHeight*(m_nPrintHdr-5), pInfo->m_rectDraw.right,m_nPrintHeight*(m_nPrintHdr-4));
	COleDateTime oleDat=COleDateTime::GetCurrentTime();
	pDC->DrawText(oleDat.Format("Vytlaen : %d.%m.%Y"), &rcText, DT_RIGHT | DT_SINGLELINE | DT_NOPREFIX | DT_END_ELLIPSIS);

	for (int ic = nCurCol; ic < m_hdr.GetItemCount(); ic++)
	{
		if(nXPos + aWidths.GetAt(ic) > pInfo->m_rectDraw.right)
			break;
		// Get width c
		memset(&hi, 0, sizeof(HD_ITEM));
		hi.pszText  = strHead.GetBuffer(255);
		hi.cchTextMax = 255;
		hi.mask = HDI_TEXT ;
		m_hdr.GetItem(ic, &hi);
		strHead.ReleaseBuffer();

		rcText.SetRect(nXPos,m_nPrintHeight*(m_nPrintHdr-3), nXPos + aWidths.GetAt(ic),m_nPrintHeight*(m_nPrintHdr));
		DrawRect(pDC,rcText);
		DrawRect(pDC,rcText+CPoint(1,1)-CSize(2,2));
		rcText.left+=5;	
		rcText.top+=5;
		rcText.right-=5;
		rcText.bottom-=5;

		pDC->DrawText(strHead, &rcText, DT_CENTER | DT_NOPREFIX | DT_END_ELLIPSIS | DT_WORDBREAK );
		nXPos += aWidths.GetAt(ic);

	}

	// print fields
	if(pInfo->m_nCurPage<=m_nPrintPages)
	{
		m_pRecSet->MoveFirst();
	}
	else
	{
		m_pRecSet->MoveFirst();
		m_pRecSet->Move
			(
			((pInfo->m_nCurPage-1)/m_nPrintPages)*m_nPrintRows
			);
//		m_pRecSet->SetAbsolutePosition
//			(
//			((pInfo->m_nCurPage-1)/m_nPrintPages)*m_nPrintRows
//			);
	}
	CString strText;
	for (int nRow = 0; nRow < m_nPrintRows; nRow++)
	{
		nXPos = nMargin;
		// Draw all columns
		for (int ic = nCurCol; ic < m_hdr.GetItemCount(); ic++)
		{
			if(nXPos + aWidths.GetAt(ic) > pInfo->m_rectDraw.right)
				break;
			// Get width
			memset(&hi, 0, sizeof(HD_ITEM));
			hi.mask = HDI_WIDTH | HDI_FORMAT;
			m_hdr.GetItem(ic, &hi);

			// Calc item rect
			CRect rcText(nXPos,(nRow+m_nPrintHdr)*m_nPrintHeight, nXPos + aWidths.GetAt(ic),(m_nPrintHdr+nRow+1)*m_nPrintHeight);
			DrawRect(pDC,rcText);
			rcText.left+=5;	
			rcText.top+=5;
			rcText.right-=5;
			rcText.bottom-=5;
			
			// Translate field value to text
			varValue = m_pRecSet->Fields->GetItem(_variant_t(m_aColumns[ic]))->Value;
			if(varValue.vt==1)
				strText="";
			else
				strText= GETSTRING(varValue);

			// Get text color
			int nAlign;
			if (hi.fmt & HDF_CENTER)
				nAlign = DT_CENTER;
			else if (hi.fmt & HDF_RIGHT)
				nAlign = DT_RIGHT;
			else
				nAlign = DT_LEFT;
			pDC->DrawText(strText, &rcText, nAlign | DT_NOPREFIX | DT_SINGLELINE | DT_END_ELLIPSIS);
			nXPos += aWidths.GetAt(ic);
		}
		m_pRecSet->MoveNext();
		if(m_pRecSet->adoEOF)
		{
			pInfo->m_bContinuePrinting = FALSE;
			break;
		}
	}
	rcText.SetRect(pInfo->m_rectDraw.left,m_nPrintHeight*(m_nPrintHdr+m_nPrintRows+1), pInfo->m_rectDraw.right,m_nPrintHeight*(m_nPrintHdr+m_nPrintRows+2));
	strHead.Format("Strana %ld (%ld/%ld)",1+(pInfo->m_nCurPage-1)/m_nPrintPages,pInfo->m_nCurPage,pInfo->GetMaxPage());
	pDC->DrawText(strHead, &rcText, DT_RIGHT | DT_SINGLELINE | DT_NOPREFIX | DT_END_ELLIPSIS);

	m_pRecSet->MoveFirst();

}
void CADOGrid::OnPrepareDC(CDC* pDC, CPrintInfo* pInfo)
{
	if(pInfo)
		if(m_pRecSet->adoEOF)
			pInfo->m_bContinuePrinting = FALSE;
}


void CADOGrid::DrawRect(CDC *pDC, CRect rc)
{
	pDC->MoveTo(rc.left,rc.top);
	pDC->LineTo(rc.left,rc.bottom);
	pDC->LineTo(rc.right,rc.bottom);
	pDC->LineTo(rc.right,rc.top);
	pDC->LineTo(rc.left,rc.top);

}

BOOL CADOGrid::OnMouseWheel(UINT nFlags, short zDelta, CPoint pt) 
{
	SCROLLINFO si;
	memset(&si, 0, sizeof(si));
	si.cbSize = sizeof(SCROLLINFO);
	si.fMask = SIF_ALL;
	GetScrollInfo(SB_VERT, &si);

	if (zDelta)
		ScrollTo(m_nScrollX,si.nPos - 5*zDelta/WHEEL_DELTA);
	
	return TRUE; //CWnd::OnMouseWheel(nFlags, zDelta, pt);
}
