#if !defined(AFX_ADOGRID_H__51741538_D858_46A2_BCE0_7692660E68FF__INCLUDED_)
#define AFX_ADOGRID_H__51741538_D858_46A2_BCE0_7692660E68FF__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
// ADOGrid.h : header file
//

/////////////////////////////////////////////////////////////////////////////
// CADOGrid window

struct NM_ADOGRID
{
	NMHDR nmhdr;
	int   iRow;
	CString strCol;
	POINT pos;
};

#define DGN_FIRST               (0U-100U)
#define DGN_ITEMCHANGED			DGN_FIRST - 0
#define DGN_DBLCLICKED			DGN_FIRST - 1
#define DGN_COLUMNCLICK			DGN_FIRST - 2
#define DGN_DIVIDERDBL			DGN_FIRST - 3
#define DGN_RCLICK				DGN_FIRST - 4

/////////////////////////////////////////////////////////////////////////////
// ADOGrid window

class CADOGrid : public CWnd
{
// Construction
public:
	CADOGrid();
	virtual ~CADOGrid();

// Attributes
public:
	int   GetColumnLeftPos(int index);
	CRect GetRowRect(int nRow);
	BOOL  IsVisible(int nRow);
	int   GetCurPos() { ASSERT(m_bBound); return m_nSelectedRow; }
	int   IsBound() { return m_bBound; }
	int   GetRecordCount() { ASSERT(m_bBound); return m_nRecCount; }
	int   GetColWidth(int nCol);
	
	long		GetFieldValueLong(const CString &strName);

// Operations
public:
	void DrawRect(CDC *pDC, CRect rc);
	void OnPrepareDC(CDC* pDC, CPrintInfo* pInfo);
	void OnPrint(CDC *pDC, CPrintInfo *pInfo);
	void OnPreparePrinting(CPrintInfo *pInfo);
	void Refresh();
	int GetColCount();

	BOOL Create(CWnd *pWndParent, int nId, int nStyle, int nExtStyle, const CRect &rect);
	void CleanUp();
	void OpenQuery(_RecordsetPtr rs, const CStringArray &aColumns,const CStringArray &aColNames);
	void ReQuery(const CString &strSql);
	void BindReQuery(BOOL nSetPos=FALSE);

	void ScrollTo(int newX, int newY);
	void EnsureVisible(int nRow);
	void SetCurPos(int nRow, BOOL bClearFindText = TRUE);
	void SetImageList(CImageList *pList);
	void SetColAlign(int nCol, int nAlign);
	void SetColWidth(int nCol, int nWidth);

	void SetContextSearch(BOOL bOn) { m_bCtxFindOn = bOn; }
	void SetContextSearchWnd(CWnd *pWnd) { ASSERT(pWnd); m_hCtxWnd = pWnd->GetSafeHwnd(); pWnd->SetWindowText(m_strCtxText); }

// Overridables
//	virtual CString OnFormatValue(int nRow, int nCol, LPCTSTR pszFieldName, 
//								  const CDBVariant &varValue, BOOL bHilightColor);
	virtual COLORREF OnFormatColor(int nRow, int nCol, LPCTSTR pszFieldName, BOOL bHilightColor);
//	virtual int OnFormatImage(int nRow, BOOL bHilightColor);

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CADOGrid)
	public:
	virtual BOOL PreTranslateMessage(MSG* pMsg);
	//}}AFX_VIRTUAL
	
// Implementation
protected:
	// Internal attributes
	CHeaderCtrl   m_hdr;
	_RecordsetPtr m_pRecSet;
	BOOL          m_bBound;
	BOOL          m_bOwner;

	CStringArray  m_aColumns;
	CImageList    *m_pImages;
	CRect         m_rcImage;
	
	int   m_nTextCy;
	int   m_nTextCx;
	int   m_nOrigY;
	CSize m_sizeGrid;

	int m_nScrollY;
	int m_nScrollX;
	int m_nSelectedRow;
	int m_nRecCount;

	// Context search support
	BOOL    m_bCtxFindOn;
	CString m_strCtxText;
	HWND    m_hCtxWnd;

	// Temporary
	CDC *m_pDC;
	int m_nLastCx;
	int m_nCx;
	int m_nCy;
	int m_nOldCx;

protected:
	void LayoutHdr();
	void CalcTextCx();
	void AdjustScrolling();
	int  FindItem(const CString &str, BOOL bDown);
/*	CDBVariant GetFieldValueInt(const CString &strName,
								short nFieldType = DEFAULT_FIELD_TYPE);
*/
protected:
	static CBrush m_brshWnd;
	static CFont  m_fnt;

	// Generated message map functions
protected:
	long m_nPrintRows;
	long m_nPrintHdr;
	long m_nPrintPages;
	CUIntArray aWidths;
	long m_nPrintHeight;
	BOOL m_isPrint;
	//{{AFX_MSG(CADOGrid)
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnPaint();
	afx_msg void OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	afx_msg void OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	afx_msg void OnSetFocus(CWnd* pOldWnd);
	afx_msg void OnKillFocus(CWnd* pNewWnd);
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnLButtonDblClk(UINT nFlags, CPoint point);
	afx_msg void OnRButtonUp(UINT nFlags, CPoint point);
	afx_msg BOOL OnMouseWheel(UINT nFlags, short zDelta, CPoint pt);
	//}}AFX_MSG
	afx_msg void OnHdrTrack(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnHdrColumnClick(NMHDR *pNMHDR, LRESULT *pResult);
	afx_msg void OnHdrDividerClick(NMHDR *pNMHDR, LRESULT *pResult);

	DECLARE_MESSAGE_MAP()
};


/*inline CString CADOGrid::OnFormatValue(int, int, LPCTSTR, const CDBVariant &varValue, 
									   BOOL)
{
	// Default implementation
	return GridVariantToString(varValue);
}

inline COLORREF CADOGrid::OnFormatColor(int, int, LPCTSTR, BOOL bHilightColor)
{
	return (bHilightColor) ? ::GetSysColor(COLOR_HIGHLIGHTTEXT) : 
	                         ::GetSysColor(COLOR_WINDOWTEXT);
}

inline int CADOGrid::OnFormatImage(int, BOOL)
{
	return 0;
}
*/

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_ADOGRID_H__51741538_D858_46A2_BCE0_7692660E68FF__INCLUDED_)
