#if !defined(AFX_DAOGRID_H__16194DB7_DF63_11D1_9034_0080ADB6E904__INCLUDED_)
#define AFX_DAOGRID_H__16194DB7_DF63_11D1_9034_0080ADB6E904__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000
// DaoGrid.h : header file
//

/////////////////////////////////////////////////////////////////////////////
// Globals
CString GridVariantToString(const COleVariant &var);

/////////////////////////////////////////////////////////////////////////////
// Notification structure
struct NM_DAOGRID
{
	NMHDR nmhdr;
	int   iRow;
};

#define DGN_FIRST               (0U-100U)
#define DGN_ITEMCHANGED			DGN_FIRST - 0
#define DGN_DBLCLICKED			DGN_FIRST - 1
#define DGN_COLUMNCLICK			DGN_FIRST - 2

/////////////////////////////////////////////////////////////////////////////
// CDaoGrid window

class CDaoGrid : public CWnd
{
// Construction
public:
	CDaoGrid();
	virtual ~CDaoGrid();

// Attributes
public:
	int   GetColumnLeftPos(int index);
	CRect GetRowRect(int nRow);
	BOOL  IsVisible(int nRow);
	int   GetCurPos() { ASSERT(m_bBound); return m_nSelectedRow; }
	int   IsBound() { return m_bBound; }
	int   GetRecordCount() { ASSERT(m_bBound); return m_pRecSet->GetRecordCount(); }
	int   GetColWidth(int nCol);
	
	COleVariant GetFieldValue(const CString &strName) { return GetFieldValue(m_nSelectedRow, strName); }
	COleVariant GetFieldValue(int nRow, const CString &strName);

// Operations
public:
	BOOL Create(CWnd *pWndParent, int nId, int nStyle, int nExtStyle, const CRect &rect);
	void OpenQuery(CDaoDatabase &db, const CString &strSql);
	void OpenQuery(CDaoDatabase &db, const CString &strSql, const CStringArray &aColumns,
		           const CStringArray &aColNames);
	void BindQuery(CDaoRecordset *rs);
	void BindQuery(CDaoRecordset *rs, const CStringArray &aColumns,
				   const CStringArray &aColNames);
	void ReQuery(const CString &strSql);
	void BindReQuery();

	void ScrollTo(int newX, int newY);
	void EnsureVisible(int nRow);
	void SetCurPos(int nRow, BOOL bClearFindText = TRUE);
	void SetImageList(CImageList *pList);
	void SetColAlign(int nCol, int nAlign);
	void SetColWidth(int nCol, int nWidth);

	void SetContextSearch(BOOL bOn) { m_bCtxFindOn = bOn; }
	void SetContextSearchWnd(CWnd *pWnd) { ASSERT(pWnd); m_hCtxWnd = pWnd->GetSafeHwnd(); pWnd->SetWindowText(m_strCtxText); }

// Overridables
	virtual CString OnFormatValue(int nRow, int nCol, LPCTSTR pszFieldName, 
								  const COleVariant &varValue, BOOL bHilightColor);
	virtual COLORREF OnFormatColor(int nRow, int nCol, LPCTSTR pszFieldName, BOOL bHilightColor);
	virtual int OnFormatImage(int nRow, BOOL bHilightColor);

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CDaoGrid)
	public:
	virtual BOOL PreTranslateMessage(MSG* pMsg);
	//}}AFX_VIRTUAL
	
// Implementation
protected:
	// Internal attributes
	CHeaderCtrl   m_hdr;
	CDaoRecordset *m_pRecSet;
	BOOL          m_bBound;
	BOOL          m_bOwner;

	CStringArray  m_aColumns;
	CImageList    *m_pImages;
	CRect         m_rcImage;
	
	int   m_nTextCy;
	int   m_nTextCx;
	int   m_nOrigY;
	CSize m_sizeGrid;

	int m_nScrollY;
	int m_nScrollX;
	int m_nSelectedRow;

	// Context search support
	BOOL    m_bCtxFindOn;
	CString m_strCtxText;
	HWND    m_hCtxWnd;

	// Temporary
	CDC *m_pDC;
	int m_nLastCx;
	int m_nCx;
	int m_nCy;
	int m_nOldCx;

protected:
	void LayoutHdr();
	void CleanUp();
	void CalcTextCx();
	void AdjustScrolling();
	int  FindItem(const CString &str, BOOL bDown);
	COleVariant GetFieldValueInt(const CString &strName);

protected:
	static CBrush m_brshWnd;
	static CFont  m_fnt;

	// Generated message map functions
protected:
	//{{AFX_MSG(CDaoGrid)
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnPaint();
	afx_msg void OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	afx_msg void OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	afx_msg void OnSetFocus(CWnd* pOldWnd);
	afx_msg void OnKillFocus(CWnd* pNewWnd);
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnLButtonDblClk(UINT nFlags, CPoint point);
	//}}AFX_MSG
	afx_msg void OnHdrTrack(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnHdrColumnClick(NMHDR *pNMHDR, LRESULT *pResult);

	DECLARE_MESSAGE_MAP()
};


inline CString CDaoGrid::OnFormatValue(int, int, LPCTSTR, const COleVariant &varValue, 
									   BOOL)
{
	// Default implementation
	return GridVariantToString(varValue);
}

inline COLORREF CDaoGrid::OnFormatColor(int, int, LPCTSTR, BOOL bHilightColor)
{
	return (bHilightColor) ? ::GetSysColor(COLOR_HIGHLIGHTTEXT) : 
	                         ::GetSysColor(COLOR_WINDOWTEXT);
}

inline int CDaoGrid::OnFormatImage(int, BOOL)
{
	return 0;
}


/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Developer Studio will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_DAOGRID_H__16194DB7_DF63_11D1_9034_0080ADB6E904__INCLUDED_)
