// OutlookPaneWnd.cpp : implementation file
//

#include "stdafx.h"
#include "OutlookPaneWnd.h"
//#include <afxpriv.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define OPW_BORDERS		4
#define OPW_SPACE		4
#define OPW_BTN		   14

/////////////////////////////////////////////////////////////////////////////
// COutlookPaneWnd
IMPLEMENT_DYNCREATE(COutlookPaneWnd, CView);

COutlookPaneWnd::COutlookPaneWnd()
{
	m_nSelected = -1;
	m_nScrollPos = 0;

	m_bUp = FALSE;
	m_bDown = FALSE;
}

COutlookPaneWnd::~COutlookPaneWnd()
{
}


BEGIN_MESSAGE_MAP(COutlookPaneWnd, CView)
	//{{AFX_MSG_MAP(COutlookPaneWnd)
	ON_WM_ERASEBKGND()
	ON_WM_MOUSEACTIVATE()
	ON_WM_CREATE()
	ON_WM_MOUSEMOVE()
	ON_WM_TIMER()
	ON_WM_SIZE()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_LBUTTONDBLCLK()
	//}}AFX_MSG_MAP
//	ON_MESSAGE(WM_HELPHITTEST, OnHelpHitTest)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// COutlookPaneWnd message handlers

BOOL COutlookPaneWnd::OnEraseBkgnd(CDC* pDC) 
{
	CRect rcInvalid;
	pDC->GetClipBox(&rcInvalid);
	pDC->FillSolidRect(&rcInvalid, ::GetSysColor(COLOR_3DSHADOW));
	
	return TRUE;
}

int COutlookPaneWnd::OnMouseActivate(CWnd* pDesktopWnd, UINT nHitTest, UINT message) 
{
	return MA_NOACTIVATE;
}

void COutlookPaneWnd::AddIcon(int nId, HICON hIcon, const CString& strText)
{
	CInfo info;
	info.m_nId = nId;
	info.m_hIcon = hIcon;
	info.m_strText = strText;

	m_listIcons.AddTail(info);
	
	// Invalidate rect
	CRect rcClient;
	GetClientRect(&rcClient);
	rcClient.top = m_nIconSize * (m_listIcons.GetCount() - m_nScrollPos - 1);
	rcClient.bottom = rcClient.top + m_nIconSize;
	InvalidateRect(&rcClient);
}

int COutlookPaneWnd::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;

	// Calculate icon size
	CClientDC dc(this);
	dc.SelectStockObject(ANSI_VAR_FONT);
	m_nIconSize = (dc.GetTextExtent("W").cy + 2) * 2 + 32 + OPW_BORDERS + OPW_SPACE;

	return 0;
}

void COutlookPaneWnd::OnDraw(CDC *pDC)
{
	CRect rcClient;
	CRect rcClip;
	GetClientRect(&rcClient);
	pDC->GetClipBox(&rcClip);
	
	pDC->SelectStockObject(ANSI_VAR_FONT);
	pDC->SetTextColor(::GetSysColor(COLOR_HIGHLIGHTTEXT));
	pDC->SetBkMode(TRANSPARENT);

	int nIconX = (rcClient.Width() - 32) / 2;

	// Draw all icons
	int i = 0;
	int index = 0;
	POSITION pos = m_listIcons.GetHeadPosition();
	while (pos)
	{
		CInfo inf = m_listIcons.GetNext(pos);

		if (index >= m_nScrollPos)
		{
			CRect rcIcon(rcClient.left, rcClient.top + i * m_nIconSize,
						 rcClient.right, rcClient.top + (i + 1) * m_nIconSize);
			CRect rcOut;
			rcOut.IntersectRect(&rcClip, &rcIcon);
			if (!rcOut.IsRectEmpty())
			{
				// Draw icon
				pDC->DrawIcon(nIconX, i * m_nIconSize + OPW_BORDERS, inf.m_hIcon);

				// Draw text
				CRect rcText(rcClient.left + OPW_BORDERS, i * m_nIconSize + OPW_BORDERS + 32 + OPW_SPACE,
							 rcClient.right - OPW_BORDERS, (i + 1) * m_nIconSize - OPW_BORDERS);
				pDC->DrawText(inf.m_strText, &rcText, DT_CENTER | DT_WORDBREAK);
			}

			i++;
		}

		index++;
	}

	// Draw buttons
	if (m_bUp)
		pDC->DrawFrameControl(CRect(rcClient.right - OPW_BTN - 1, rcClient.top + 1, rcClient.right - 1, rcClient.top + OPW_BTN + 1),
							  DFC_SCROLL, DFCS_SCROLLUP); 

	if (m_bDown)
		pDC->DrawFrameControl(CRect(rcClient.right - OPW_BTN - 1, rcClient.bottom - OPW_BTN - 1, rcClient.right - 1, rcClient.bottom - 1),
							  DFC_SCROLL, DFCS_SCROLLDOWN); 
}

void COutlookPaneWnd::OnMouseMove(UINT nFlags, CPoint point) 
{
	CRect rcClient;
	GetClientRect(&rcClient);
	int nIconX = (rcClient.Width() - 32) / 2;
	
	CClientDC dc(this);

	int newSel = -1;
	CRect rcSel;

	for (int i = 0; i < m_listIcons.GetCount(); i++)
	{
		CRect rcIcon(nIconX, (i - m_nScrollPos) * m_nIconSize + OPW_BORDERS,
					 nIconX + 32, (i - m_nScrollPos) * m_nIconSize + OPW_BORDERS + 32);
		if (rcIcon.PtInRect(point))
		{
			newSel = i + m_nScrollPos;
			rcSel = rcIcon;
			break;
		}
	}

	if (newSel != m_nSelected)
	{
		// Draw new border
		if (newSel != -1)
		{
			rcSel.InflateRect(3, 3);
			dc.Draw3dRect(&rcSel, ::GetSysColor(COLOR_3DFACE), 0);
		}

		// Erase old
		if (m_nSelected != -1)
		{
			CPen pen(PS_SOLID, 1, ::GetSysColor(COLOR_3DSHADOW));
			CPen *pOld = dc.SelectObject(&pen);
			dc.SelectStockObject(NULL_BRUSH);
			rcSel.SetRect(nIconX, (m_nSelected - m_nScrollPos) * m_nIconSize + OPW_BORDERS,
						  nIconX + 32, (m_nSelected - m_nScrollPos) * m_nIconSize + OPW_BORDERS + 32);
			rcSel.InflateRect(3, 3);
			dc.Rectangle(rcSel);
			dc.SelectObject(pOld);
		}

		m_nSelected = newSel;
	}

	// Set timer for clearing active button
	if (newSel != -1)
		SetTimer(100, 2000, 0);
}

void COutlookPaneWnd::OnTimer(UINT nIDEvent) 
{
	if (nIDEvent == 100)
	{
		KillTimer(100);
		CPoint pt;
		::GetCursorPos(&pt);
		ScreenToClient(&pt);
		OnMouseMove(0, pt);
	}
	
	CView::OnTimer(nIDEvent);
}

void COutlookPaneWnd::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
	UpdateButtons();
}

BOOL COutlookPaneWnd::PreCreateWindow(CREATESTRUCT& cs) 
{
	cs.style |= WS_CLIPCHILDREN;
	return CView::PreCreateWindow(cs);
}

void COutlookPaneWnd::UpdateButtons()
{
	CRect rcClient;
	int   nLow = (m_listIcons.GetCount() - m_nScrollPos) * m_nIconSize;
	GetClientRect(rcClient);
	
	m_bUp = m_nScrollPos;
	InvalidateRect(CRect(rcClient.right - OPW_BTN - 1, rcClient.top,
						 rcClient.right - 1, rcClient.top + OPW_BTN));
	
    m_bDown = (nLow > rcClient.bottom) && (m_nScrollPos < m_listIcons.GetCount() - 1);
	InvalidateRect(CRect(rcClient.right - OPW_BTN - 1, rcClient.bottom - OPW_BTN - 1,
						 rcClient.right - 1, rcClient.bottom - 1));
}

void COutlookPaneWnd::OnLButtonDown(UINT nFlags, CPoint point) 
{
	CRect rcClient;
	GetClientRect(&rcClient);
	
	CRect rcUp(rcClient.right - OPW_BTN - 1, rcClient.top + 1, rcClient.right - 1, rcClient.top + OPW_BTN + 1);
	CRect rcDown(rcClient.right - OPW_BTN - 1, rcClient.bottom - OPW_BTN - 1, rcClient.right - 1, rcClient.bottom - 1);

	CClientDC dc(this);

	// Up button
	if (m_bUp && rcUp.PtInRect(point))
	{
		dc.FillSolidRect(&rcUp, ::GetSysColor(COLOR_3DSHADOW));
		dc.FillSolidRect(&rcDown, ::GetSysColor(COLOR_3DSHADOW));
		ASSERT(m_nScrollPos);
		Scroll(-1);
		InvalidateRect(rcUp);
		InvalidateRect(rcDown);
		UpdateWindow();
		dc.DrawFrameControl(rcUp, DFC_SCROLL, DFCS_SCROLLUP | DFCS_PUSHED); 
		SetCapture();
		while (GetCapture() == this)
		{
		    MSG msg;
			// Handle dialog messages
			while(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
			{
				TranslateMessage(&msg);
				DispatchMessage(&msg);  
			}
		}
		InvalidateRect(rcUp);
		return;
	}

	// Down button
	if (m_bDown && rcDown.PtInRect(point))
	{
		dc.FillSolidRect(&rcUp, ::GetSysColor(COLOR_3DSHADOW));
		dc.FillSolidRect(&rcDown, ::GetSysColor(COLOR_3DSHADOW));
		ASSERT(m_nScrollPos < m_listIcons.GetCount() - 1);
		Scroll(1);
		InvalidateRect(rcUp);
		InvalidateRect(rcDown);
		UpdateWindow();
		dc.DrawFrameControl(rcDown, DFC_SCROLL, DFCS_SCROLLDOWN | DFCS_PUSHED); 
		SetCapture();
		while (GetCapture() == this)
		{
		    MSG msg;
			// Handle dialog messages
			while(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
			{
				TranslateMessage(&msg);
				DispatchMessage(&msg);  
			}
		}
		InvalidateRect(rcDown);
		return;
	}

	// Check icon hit
	int nIconX = (rcClient.Width() - 32) / 2;

	int newSel = -1;
	int nId;
	CRect rcSel;

	POSITION pos = m_listIcons.GetHeadPosition();
	int i = 0;
	while (pos)
	{
		CInfo inf = m_listIcons.GetNext(pos);

		CRect rcIcon(nIconX, (i - m_nScrollPos) * m_nIconSize + OPW_BORDERS,
					 nIconX + 32, (i - m_nScrollPos) * m_nIconSize + OPW_BORDERS + 32);
		if (rcIcon.PtInRect(point))
		{
			newSel = i + m_nScrollPos;
			rcSel = rcIcon;
			nId = inf.m_nId;
			break;
		}

		i++;
	}

	// Draw new border - sunken
	if (newSel != -1)
	{
		rcSel.InflateRect(3, 3);
		dc.Draw3dRect(&rcSel, 0, ::GetSysColor(COLOR_3DFACE));
	}

	// Erase old
	if (m_nSelected != -1 && m_nSelected != newSel)
	{
		CPen pen(PS_SOLID, 1, ::GetSysColor(COLOR_3DSHADOW));
		CPen *pOld = dc.SelectObject(&pen);
		dc.SelectStockObject(NULL_BRUSH);
		rcSel.SetRect(nIconX, (m_nSelected - m_nScrollPos) * m_nIconSize + OPW_BORDERS,
					  nIconX + 32, (m_nSelected - m_nScrollPos) * m_nIconSize + OPW_BORDERS + 32);
		rcSel.InflateRect(3, 3);
		dc.Rectangle(rcSel);
		dc.SelectObject(pOld);
	}

	m_nSelected = newSel;
	if (newSel != -1)
	{
		SetTimer(100, 2000, 0);

		// Fire event
		CWnd* pWndParent = GetParent();
		if (pWndParent)
			pWndParent->SendMessage(WM_OPW_SELCHANGE, nId);
	}
}

void COutlookPaneWnd::Scroll(int dy)
{
	m_nScrollPos += dy;
	if (m_nScrollPos < 0) m_nScrollPos = 0;
	if (m_nScrollPos >= m_listIcons.GetCount() - 1) m_nScrollPos = m_listIcons.GetCount() - 1;

	// Scroll window
	ScrollWindow(0, (-dy) * m_nIconSize);
	UpdateButtons();
}

void COutlookPaneWnd::OnLButtonUp(UINT nFlags, CPoint point) 
{
	if (GetCapture() == this)
		ReleaseCapture();
}

void COutlookPaneWnd::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
	OnLButtonDown(nFlags, point);
}

CString COutlookPaneWnd::GetInfoText(int nId)
{
	POSITION pos = m_listIcons.GetHeadPosition();
	int i = 0;
	while (pos)
	{
		CInfo inf = m_listIcons.GetNext(pos);
		if (inf.m_nId == nId)
			return inf.m_strText;
	}

	return CString();
}

LRESULT COutlookPaneWnd::OnHelpHitTest(WPARAM, LPARAM)
{
	return HIDW_OUTLOOK_PANE;
}

void COutlookPaneWnd::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint)
{
}