// OutlookSplitterWnd.cpp : implementation file
//

#include "stdafx.h"
#include "OutlookSplitterWnd.h"
#include "OutlookPaneWnd.h"
#include <afxpriv.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// Note: afxData.cxBorder and afxData.cyBorder aren't used anymore
#define CX_BORDER   1
#define CY_BORDER   1

struct AUX_DATA
{
	// system metrics
	int cxVScroll, cyHScroll;
	int cxIcon, cyIcon;

	int cxBorder2, cyBorder2;

	// device metrics for screen
	int cxPixelsPerInch, cyPixelsPerInch;

	// solid brushes with convenient gray colors and system colors
	HBRUSH hbrLtGray, hbrDkGray;
	HBRUSH hbrBtnHilite, hbrBtnFace, hbrBtnShadow;
	HBRUSH hbrWindowFrame;
#ifdef _MAC
	HBRUSH hbr3DLight;
#endif
	HPEN hpenBtnHilite, hpenBtnShadow, hpenBtnText;

	// color values of system colors used for CToolBar
	COLORREF clrBtnFace, clrBtnShadow, clrBtnHilite;
	COLORREF clrBtnText, clrWindowFrame;
#ifdef _MAC
	COLORREF clr3DLight;
#endif

	// standard cursors
	HCURSOR hcurWait;
	HCURSOR hcurArrow;
	HCURSOR hcurHelp;       // cursor used in Shift+F1 help

	// special GDI objects allocated on demand
	HFONT   hStatusFont;
	HFONT   hToolTipsFont;
	HBITMAP hbmMenuDot;

	// other system information
	UINT    nWinVer;        // Major.Minor version numbers
	BOOL    bWin32s;        // TRUE if Win32s (or Windows 95)
	BOOL    bWin4;          // TRUE if Windows 4.0
	BOOL    bNotWin4;       // TRUE if not Windows 4.0
	BOOL    bSmCaption;     // TRUE if WS_EX_SMCAPTION is supported
	BOOL    bWin31;         // TRUE if actually Win32s on Windows 3.1
	BOOL    bMarked4;       // TRUE if marked as 4.0

#ifdef _MAC
	BOOL    bOleIgnoreSuspend;
#endif

// Implementation
	AUX_DATA();
	~AUX_DATA();
	void UpdateSysColors();
	void UpdateSysMetrics();
};

extern AFX_CORE_DATA AUX_DATA afxData;


/////////////////////////////////////////////////////////////////////////////
// COutlookSplitterWnd

COutlookSplitterWnd::COutlookSplitterWnd()
{
	m_nInfoHeight = 0;
}

COutlookSplitterWnd::~COutlookSplitterWnd()
{
}


BEGIN_MESSAGE_MAP(COutlookSplitterWnd, CSplitterWnd)
	//{{AFX_MSG_MAP(COutlookSplitterWnd)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_MSG_MAP
	ON_WM_CTLCOLOR()
	ON_MESSAGE(WM_OPW_SELCHANGE, OnSwitchView)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// Pane shifting
// repositions client area of specified window
// assumes everything has WS_BORDER or is inset like it does
//  (includes scroll bars)
static void  DeferClientPos(AFX_SIZEPARENTPARAMS* lpLayout,
	CWnd* pWnd, int x, int y, int cx, int cy, BOOL bScrollBar)
{
	ASSERT(pWnd != NULL);
	ASSERT(pWnd->m_hWnd != NULL);

	if (bScrollBar)
	{
		// if there is enough room, draw scroll bar without border
		// if there is not enough room, set the WS_BORDER bit so that
		//   we will at least get a proper border drawn
		BOOL bNeedBorder = (cx <= CX_BORDER || cy <= CY_BORDER);
		pWnd->ModifyStyle(bNeedBorder ? 0 : WS_BORDER,
			bNeedBorder ? WS_BORDER : 0);
	}
	CRect rect(x, y, x+cx, y+cy);

	// adjust for border size (even if zero client size)
	if (!afxData.bWin4)
	{
		if (bScrollBar)
			rect.InflateRect(CX_BORDER, CY_BORDER);
		else
			pWnd->CalcWindowRect(&rect);
	}

	// adjust for 3d border (splitter windows have implied border)
	if ((pWnd->GetExStyle() & WS_EX_CLIENTEDGE) ||
		  pWnd->IsKindOf(RUNTIME_CLASS(CSplitterWnd)))
		rect.InflateRect(afxData.cxBorder2, afxData.cyBorder2);

	// first check if the new rectangle is the same as the current
	CRect rectOld;
	pWnd->GetWindowRect(rectOld);
	pWnd->GetParent()->ScreenToClient(&rectOld);
	if (rect != rectOld)
		AfxRepositionWindow(lpLayout, pWnd->m_hWnd, rect);
}

// Generic routine:
//  for X direction: pInfo = m_pColInfo, nMax = m_nMaxCols, nSize = cx
//  for Y direction: pInfo = m_pRowInfo, nMax = m_nMaxRows, nSize = cy
static void  LayoutRowCol(CSplitterWnd::CRowColInfo* pInfoArray,
	int nMax, int nSize, int nSizeSplitter)
{
	ASSERT(pInfoArray != NULL);
	ASSERT(nMax > 0);
	ASSERT(nSizeSplitter > 0);

	CSplitterWnd::CRowColInfo* pInfo;
	int i;

	if (nSize < 0)
		nSize = 0;  // if really too small, layout as zero size

	// start with ideal sizes
	for (i = 0, pInfo = pInfoArray; i < nMax-1; i++, pInfo++)
	{
		if (pInfo->nIdealSize < pInfo->nMinSize)
			pInfo->nIdealSize = 0;      // too small to see
		pInfo->nCurSize = pInfo->nIdealSize;
	}
	pInfo->nCurSize = INT_MAX;  // last row/column takes the rest

	for (i = 0, pInfo = pInfoArray; i < nMax; i++, pInfo++)
	{
		ASSERT(nSize >= 0);
		if (nSize == 0)
		{
			// no more room (set pane to be invisible)
			pInfo->nCurSize = 0;
			continue;       // don't worry about splitters
		}
		else if (nSize < pInfo->nMinSize && i != 0)
		{
			// additional panes below the recommended minimum size
			//   aren't shown and the size goes to the previous pane
			pInfo->nCurSize = 0;

			// previous pane already has room for splitter + border
			//   add remaining size and remove the extra border
			ASSERT(afxData.cxBorder2 == afxData.cyBorder2);
			(pInfo-1)->nCurSize += nSize + afxData.cxBorder2;
			nSize = 0;
		}
		else
		{
			// otherwise we can add the second pane
			ASSERT(nSize > 0);
			if (pInfo->nCurSize == 0)
			{
				// too small to see
				if (i != 0)
					pInfo->nCurSize = 0;
			}
			else if (nSize < pInfo->nCurSize)
			{
				// this row/col won't fit completely - make as small as possible
				pInfo->nCurSize = nSize;
				nSize = 0;
			}
			else
			{
				// can fit everything
				nSize -= pInfo->nCurSize;
			}
		}

		// see if we should add a splitter
		ASSERT(nSize >= 0);
		if (i != nMax - 1)
		{
			// should have a splitter
			if (nSize > nSizeSplitter)
			{
				nSize -= nSizeSplitter; // leave room for splitter + border
				ASSERT(nSize > 0);
			}
			else
			{
				// not enough room - add left over less splitter size
				ASSERT(afxData.cxBorder2 == afxData.cyBorder2);
				pInfo->nCurSize += nSize;
				if (pInfo->nCurSize > (nSizeSplitter - afxData.cxBorder2))
					pInfo->nCurSize -= (nSizeSplitter - afxData.cyBorder2);
				nSize = 0;
			}
		}
	}
	ASSERT(nSize == 0); // all space should be allocated
}

void COutlookSplitterWnd::RecalcLayout()
{
	ASSERT_VALID(this);
	ASSERT(m_nRows > 0 && m_nCols > 0); // must have at least one pane

	CRect rectClient;
	GetClientRect(rectClient);
	rectClient.InflateRect(-m_cxBorder, -m_cyBorder);

	CRect rectInside;
	GetInsideRect(rectInside);

	// layout columns (restrict to possible sizes)
	LayoutRowCol(m_pColInfo, m_nCols, rectInside.Width(), m_cxSplitterGap);
	LayoutRowCol(m_pRowInfo, m_nRows, rectInside.Height(), m_cySplitterGap);

	// adjust the panes (and optionally scroll bars)

	// give the hint for the maximum number of HWNDs
	AFX_SIZEPARENTPARAMS layout;
	layout.hDWP = ::BeginDeferWindowPos((m_nCols + 1) * (m_nRows + 1) + 1);

	// size of scrollbars
	int cx = (rectClient.right - rectInside.right) - afxData.bNotWin4;
	int cy = (rectClient.bottom - rectInside.bottom) - afxData.bNotWin4;

	// reposition size box
	if (m_bHasHScroll && m_bHasVScroll)
	{
		CWnd* pScrollBar = GetDlgItem(AFX_IDW_SIZE_BOX);
		ASSERT(pScrollBar != NULL);

		// fix style if necessary
		BOOL bSizingParent = (GetSizingParent() != NULL);
		// modifyStyle returns TRUE if style changes
		if (pScrollBar->ModifyStyle(SBS_SIZEGRIP|SBS_SIZEBOX,
				bSizingParent ? SBS_SIZEGRIP : SBS_SIZEBOX))
			pScrollBar->Invalidate();
		pScrollBar->EnableWindow(bSizingParent);

		// reposition the size box
		DeferClientPos(&layout, pScrollBar,
			rectInside.right + afxData.bNotWin4,
			rectInside.bottom + afxData.bNotWin4, cx, cy, TRUE);
	}

	// reposition scroll bars
	if (m_bHasHScroll)
	{
		int cxSplitterBox = m_cxSplitter + afxData.bNotWin4;// split box bigger
		int x = rectClient.left;
		int y = rectInside.bottom + afxData.bNotWin4;
		for (int col = 0; col < m_nCols; col++)
		{
			CWnd* pScrollBar = GetDlgItem(AFX_IDW_HSCROLL_FIRST + col);
			ASSERT(pScrollBar != NULL);
			int cx = m_pColInfo[col].nCurSize;
			if (col == 0 && m_nCols < m_nMaxCols)
				x += cxSplitterBox, cx -= cxSplitterBox;
			DeferClientPos(&layout, pScrollBar, x, y, cx, cy, TRUE);
			x += cx + m_cxSplitterGap;
		}
	}

	if (m_bHasVScroll)
	{
		int cySplitterBox = m_cySplitter + afxData.bNotWin4;// split box bigger
		int x = rectInside.right + afxData.bNotWin4;
		int y = rectClient.top;
		for (int row = 0; row < m_nRows; row++)
		{
			CWnd* pScrollBar = GetDlgItem(AFX_IDW_VSCROLL_FIRST + row);
			ASSERT(pScrollBar != NULL);
			int cy = m_pRowInfo[row].nCurSize;
			if (row == 0 && m_nRows < m_nMaxRows)
				y += cySplitterBox, cy -= cySplitterBox;
			DeferClientPos(&layout, pScrollBar, x, y, cx, cy, TRUE);
			y += cy + m_cySplitterGap;
		}
	}

	//BLOCK: Reposition all the panes
	{
		int x = rectClient.left;
		for (int col = 0; col < m_nCols; col++)
		{
			int cx = m_pColInfo[col].nCurSize;
			int y = rectClient.top + ((col == 1) ? m_nInfoHeight : 0);
			for (int row = 0; row < m_nRows; row++)
			{
				int cy = m_pRowInfo[row].nCurSize - ((col == 1) ? m_nInfoHeight : 0);
				CWnd* pWnd = GetPane(row, col);
				DeferClientPos(&layout, pWnd, x, y, cx, cy, FALSE);

				if (col == 1)
					m_stInfo.MoveWindow(x, rectClient.top, cx, y - rectClient.top);
				
				y += cy + m_cySplitterGap;
			}
			x += cx + m_cxSplitterGap;
		}
	}

	// move and resize all the windows at once!
	if (layout.hDWP == NULL || !::EndDeferWindowPos(layout.hDWP))
		TRACE0("Warning: DeferWindowPos failed - low system resources.\n");

	// invalidate all the splitter bars (with NULL pDC)
	DrawAllSplitBars(NULL, rectInside.right, rectInside.bottom);
}



/////////////////////////////////////////////////////////////////////////////
// COutlookSplitterWnd message handlers
BOOL COutlookSplitterWnd::Create(CWnd* pWndParent, UINT nIconId, UINT nStringId,
								 CCreateContext* pViewContext)
{
	if (!CreateStatic(pWndParent, 1, 2, WS_CHILD | WS_VISIBLE | WS_BORDER))
		return FALSE;

	if (!CreateView(0, 0, RUNTIME_CLASS(COutlookPaneWnd), CSize(100, 0), pViewContext))
		return FALSE;

	if (!CreateView(0, 1, pViewContext->m_pNewViewClass, CSize(100, 0), pViewContext))
		return FALSE;

	// Create info
	if (!m_stInfo.Create(NULL, WS_CHILD | WS_VISIBLE | SS_LEFT, CRect(0,0,0,0), this, 11))
		return FALSE;

	// Create font
	LOGFONT lfn;
	memset(&lfn, 0, sizeof(LOGFONT));
	lfn.lfCharSet = 0xEE;  // Central european
	lfn.lfHeight = 160;
	lstrcpy(lfn.lfFaceName, _T("Times New Roman"));
	lfn.lfWeight = FW_NORMAL;
	m_fntInfo.CreatePointFontIndirect(&lfn);
	
	// Get info
	{
		CClientDC dc(&m_stInfo);
		CFont* pOld = dc.SelectObject(&m_fntInfo);
		m_nInfoHeight = dc.GetTextExtent("W").cy + 2;
		dc.SelectObject(pOld);
	}

	m_brshInfo.CreateSolidBrush(::GetSysColor(COLOR_3DSHADOW));
	m_stInfo.SetFont(&m_fntInfo);
	SetActivePane(0,1);

	// Add first icon
	CString strInfo((LPCSTR)nStringId);
	((COutlookPaneWnd*)GetPane(0,0))->AddIcon(nIconId, AfxGetApp()->LoadIcon(nIconId), 
										      strInfo);
	m_stInfo.SetWindowText(" " + strInfo);
	m_nCurrentId = nIconId;

	// Add first view to map
	m_mapViews.SetAt(nIconId, (CView*)GetPane(0, 1));

	return TRUE;
}

BOOL COutlookSplitterWnd::Create(CWnd* pWndParent, UINT nId, HICON hIcon, 
								 LPCTSTR lpszString, CCreateContext* pViewContext)
{
	if (!CreateStatic(pWndParent, 1, 2, WS_CHILD | WS_VISIBLE | WS_BORDER))
		return FALSE;

	if (!CreateView(0, 0, RUNTIME_CLASS(COutlookPaneWnd), CSize(100, 0), pViewContext))
		return FALSE;

	if (!CreateView(0, 1, pViewContext->m_pNewViewClass, CSize(100, 0), pViewContext))
		return FALSE;

	// Create info
	if (!m_stInfo.Create(NULL, WS_CHILD | WS_VISIBLE | SS_LEFT, CRect(0,0,0,0), this, 11))
		return FALSE;

	// Create font
	LOGFONT lfn;
	memset(&lfn, 0, sizeof(LOGFONT));
	lfn.lfCharSet = 0xEE;  // Central european
	lfn.lfHeight = 160;
	lstrcpy(lfn.lfFaceName, _T("Times New Roman"));
	lfn.lfWeight = FW_NORMAL;
	m_fntInfo.CreatePointFontIndirect(&lfn);
	
	// Get info
	{
		CClientDC dc(&m_stInfo);
		CFont* pOld = dc.SelectObject(&m_fntInfo);
		m_nInfoHeight = dc.GetTextExtent("W").cy + 2;
		dc.SelectObject(pOld);
	}

	m_brshInfo.CreateSolidBrush(::GetSysColor(COLOR_3DSHADOW));
	m_stInfo.SetFont(&m_fntInfo);

	// Add first icon
	((COutlookPaneWnd*)GetPane(0,0))->AddIcon(nId, hIcon, lpszString);
	m_stInfo.SetWindowText(" " + CString(lpszString));
	m_nCurrentId = nId;

	// Add first view to map
	m_mapViews.SetAt(nId, (CView*)GetPane(0, 1));
	SetActivePane(0,1);

	return TRUE;
}

HBRUSH COutlookSplitterWnd::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor)
{
	if (nCtlColor == CTLCOLOR_STATIC)
	{
		pDC->SetTextColor(GetSysColor(COLOR_HIGHLIGHTTEXT));
		pDC->SetBkColor(GetSysColor(COLOR_3DSHADOW));
		return m_brshInfo;
	}
	else
		return CWnd::OnCtlColor(pDC, pWnd, nCtlColor);
}

BOOL COutlookSplitterWnd::AddView(UINT nIconId, UINT nStringId, CRuntimeClass *pViewClass,
								  BOOL bSendInitialUpdate)
{
#ifdef _DEBUG
	ASSERT_VALID(this);
	ASSERT(pViewClass != NULL);
	ASSERT(pViewClass->IsDerivedFrom(RUNTIME_CLASS(CView)));
	ASSERT(AfxIsValidAddress(pViewClass, sizeof(CRuntimeClass), FALSE));
#endif

	CCreateContext contextT;
	CView* pOldView = (CView*)GetActivePane();
	if (pOldView != NULL && pOldView->IsKindOf(RUNTIME_CLASS(CView)))
	{
		// set info about last pane
		ASSERT(contextT.m_pCurrentFrame == NULL);
		contextT.m_pLastView = pOldView;
		contextT.m_pCurrentDoc = pOldView->GetDocument();
		if (contextT.m_pCurrentDoc != NULL)
			contextT.m_pNewDocTemplate =
			  contextT.m_pCurrentDoc->GetDocTemplate();
	}
	CCreateContext* pContext = &contextT;

	CWnd* pWnd;
	TRY
	{
		pWnd = (CWnd*)pViewClass->CreateObject();
		if (pWnd == NULL)
			AfxThrowMemoryException();
	}
	CATCH_ALL(e)
	{
		TRACE0("Out of memory creating a splitter pane.\n");
		// Note: DELETE_EXCEPTION(e) not required
		return FALSE;
	}
	END_CATCH_ALL

	ASSERT_KINDOF(CWnd, pWnd);
	ASSERT(pWnd->m_hWnd == NULL);       // not yet created

	DWORD dwStyle = AFX_WS_DEFAULT_VIEW;
	dwStyle &= ~(WS_BORDER | WS_VISIBLE);

	// Create with the right size (wrong position)
	CRect rect(CPoint(0,0), CSize(100,100));
	if (!pWnd->Create(NULL, NULL, dwStyle,
		rect, this, m_mapViews.GetCount() + 1, pContext))
	{
		TRACE0("Warning: couldn't create client pane for splitter.\n");
			// pWnd will be cleaned up by PostNcDestroy
		return FALSE;
	}

	if (bSendInitialUpdate)
		pWnd->SendMessage(WM_INITIALUPDATE);
	pWnd->EnableWindow(FALSE);

	// Add icon
	CString strInfo((LPCSTR)nStringId);
	((COutlookPaneWnd*)GetPane(0,0))->AddIcon(nIconId, AfxGetApp()->LoadIcon(nIconId), 
										      strInfo);
	// Add view to map
	m_mapViews.SetAt(nIconId, (CView*)pWnd);
	return TRUE;
}

BOOL COutlookSplitterWnd::AddView(UINT nId, HICON hIcon, LPCTSTR lpszString, 
								  CRuntimeClass *pViewClass, BOOL bSendInitialUpdate)
{
#ifdef _DEBUG
	ASSERT_VALID(this);
	ASSERT(pViewClass != NULL);
	ASSERT(pViewClass->IsDerivedFrom(RUNTIME_CLASS(CView)));
	ASSERT(AfxIsValidAddress(pViewClass, sizeof(CRuntimeClass), FALSE));
#endif

	CCreateContext contextT;
	CView* pOldView = (CView*)GetActivePane();
	if (pOldView != NULL && pOldView->IsKindOf(RUNTIME_CLASS(CView)))
	{
		// set info about last pane
		ASSERT(contextT.m_pCurrentFrame == NULL);
		contextT.m_pLastView = pOldView;
		contextT.m_pCurrentDoc = pOldView->GetDocument();
		if (contextT.m_pCurrentDoc != NULL)
			contextT.m_pNewDocTemplate =
			  contextT.m_pCurrentDoc->GetDocTemplate();
	}
	CCreateContext* pContext = &contextT;

	CWnd* pWnd;
	TRY
	{
		pWnd = (CWnd*)pViewClass->CreateObject();
		if (pWnd == NULL)
			AfxThrowMemoryException();
	}
	CATCH_ALL(e)
	{
		TRACE0("Out of memory creating a splitter pane.\n");
		// Note: DELETE_EXCEPTION(e) not required
		return FALSE;
	}
	END_CATCH_ALL

	ASSERT_KINDOF(CWnd, pWnd);
	ASSERT(pWnd->m_hWnd == NULL);       // not yet created

	DWORD dwStyle = AFX_WS_DEFAULT_VIEW;
	dwStyle &= ~(WS_BORDER | WS_VISIBLE);

	// Create with the right size (wrong position)
	CRect rect(CPoint(0,0), CSize(100,100));
	if (!pWnd->Create(NULL, NULL, dwStyle,
		rect, this, m_mapViews.GetCount() + 1, pContext))
	{
		TRACE0("Warning: couldn't create client pane for splitter.\n");
			// pWnd will be cleaned up by PostNcDestroy
		return FALSE;
	}

	if (bSendInitialUpdate)
		pWnd->SendMessage(WM_INITIALUPDATE);
	pWnd->EnableWindow(FALSE);

	// Add icon
	((COutlookPaneWnd*)GetPane(0,0))->AddIcon(nId, hIcon, lpszString);
	// Add view to map
	m_mapViews.SetAt(nId, (CView*)pWnd);
	return TRUE;
}

void COutlookSplitterWnd::SwitchToView(UINT nIconId)
{
	if ((int)nIconId == m_nCurrentId)
		return;

	m_nCurrentId = nIconId;
	CView* pOld = (CView*)GetPane(0, 1);
	CView* pNew;
	BOOL   bRes = m_mapViews.Lookup(nIconId, pNew);
	ASSERT(bRes);

	// Show new view
	int nOld = pNew->SetDlgCtrlID(IdFromRowCol(0, 1));
	ASSERT(nOld);
	pOld->SetDlgCtrlID(nOld);
	pOld->ShowWindow(SW_HIDE);
	pOld->EnableWindow(FALSE);

	pNew->EnableWindow(TRUE);
	RecalcLayout();
	pNew->ShowWindow(SW_SHOW);
	pNew->UpdateWindow();
	SetActivePane(0, 1);

	m_stInfo.SetWindowText(" " + ((COutlookPaneWnd*)GetPane(0, 0))->GetInfoText(nIconId));
}

afx_msg LRESULT COutlookSplitterWnd::OnSwitchView(WPARAM wParam, LPARAM)
{
	SwitchToView(wParam);
	return 0;
}